@@ -113,6 +113,7 @@ cut_utils_inspect_memory (const void *memory, size_t size)
     const guchar *binary = memory;
     GString *buffer;
     size_t i, n_printable_characters;
+    size_t max_size = 1024;
 
     if (memory == NULL || size == 0)
         return g_strdup("(null)");
@@ -121,22 +122,27 @@ cut_utils_inspect_memory (const void *memory, size_t size)
                                 (size - 1) * strlen(" ") +
                                 strlen(": ") +
                                 size);
+    max_size = MIN(size, max_size);
     n_printable_characters = 0;
-    for (i = 0; i < size; i++) {
+    for (i = 0; i < max_size; i++) {
         g_string_append_printf(buffer, "0x%02x ", binary[i]);
         if (g_ascii_isprint(binary[i]))
             n_printable_characters++;
     }
+    if (size > max_size)
+        g_string_append(buffer, "... ");
 
-    if (n_printable_characters >= size * 0.3) {
+    if (n_printable_characters >= max_size * 0.3) {
         g_string_overwrite(buffer, buffer->len - 1, ": ");
-        for (i = 0; i < size; i++) {
+        for (i = 0; i < max_size; i++) {
             if (g_ascii_isprint(binary[i])) {
                 g_string_append_c(buffer, binary[i]);
             } else {
                 g_string_append_c(buffer, '.');
             }
         }
+        if (size > max_size)
+            g_string_append(buffer, "...");
     } else {
         g_string_truncate(buffer, buffer->len - 1);
     }
