@@ -5,12 +5,17 @@ require "optparse"
 require "rd/rdvisitor"
 require "rd/version"
 
+require 'rd-visitor-util'
+
+require "rt/rtparser"
+
 require "gettext"
 require "gettext/poparser"
 
 module RD
   class RD2RefEntryVisitor < RDVisitor
     include ERB::Util
+    include RDVisitorUtil
 
     SYSTEM_NAME = "RDtool -- RD2RefEntryVisitor"
     SYSTEM_VERSION = "0.0.1"
@@ -43,6 +48,7 @@ module RD
       @translate_data = nil
       @n_sections = 0
       @n_text_blocks_in_section = 0
+      @term_indexes = {}
       @man_style = false
     end
 
@@ -59,6 +65,11 @@ module RD
     end
 
     def apply_to_Headline(element, title)
+      id = nil
+      if /\A\s*\[(.+?)\]\s*/ =~ title.first
+        id = $1
+        title[0] = $POSTMATCH
+      end
       case element.level
       when 1
         title = title.join
@@ -75,15 +86,55 @@ module RD
         @n_sections += 1
         @n_text_blocks_in_section = 0
       end
-      [:headline, element.level - 1, title]
+      [:headline, element.level - 1, title, id]
     end
 
     def apply_to_Verbatim(element)
-      contents = []
+      lines = []
       element.each_line do |line|
-	contents.push(apply_to_String(line))
+        lines.push(line)
       end
-      tag("programlisting", {}, contents.join("").chomp)
+
+      case lines.first.strip
+      when "# RT"
+        apply_to_RT(lines.join)
+      else
+        contents = lines.collect {|line| apply_to_String(line)}
+        tag("programlisting", {}, contents.join("").chomp)
+      end
+    end
+
+    def apply_to_RT(content)
+      rt = RT::RTParser.parse(content)
+      elements = [tag("caption", {}, apply_to_String(rt.config["caption"]))]
+
+      header_rows = []
+      rt.header.each do |row|
+        header_cells = []
+        row.each do |cell|
+          if cell.class == RT::RTCell
+            header_cells << [tag("th", {}, apply_to_String(cell.value))]
+          end
+        end
+        header_rows << [tag("tr", {}, header_cells)]
+      end
+      elements << [tag("thead", {}, header_rows)]
+
+      rows = []
+      rt.body.each do |row|
+        cells = []
+        row.each do |cell|
+          if cell.class == RT::RTCell
+            cells << [tag("td", {}, apply_to_String(cell.value))]
+          end
+        end
+        rows << [tag("tr", {}, cells)]
+      end
+      elements << [tag("tbody", {}, rows)]
+
+      attributes = {}
+      attributes["id"] = rt.config["id"] if rt.config["id"]
+      tag("table", attributes, elements)
     end
 
     def apply_to_Reference_with_RDLabel(element, contents)
@@ -114,9 +165,20 @@ module RD
                     tag("textobject", {}, tag("phrase", {}, label)),
                    ])
       else
+        if url =~ /#/
+          target, hash = $PREMATCH, $POSTMATCH
+          if target.empty?
+            linkend = term_index_id(hash)
+          else
+            target = nil if target == "."
+            linkend = "#{ref_entry_id(target)}.#{term_index_id(hash)}"
+          end
+        else
+          linkend = ref_entry_id(url)
+        end
         return tag("link",
-                   {:linkend => ref_entry_id(url)},
-                   remove_suffix(remove_lang_suffix(label)))
+                   {:linkend => linkend},
+                   remove_suffix(remove_lang_suffix(remove_hash_suffix(label))))
       end
       label = remove_suffix(remove_lang_suffix(label))
       tag("ulink", {:url => url}, label)
@@ -150,8 +212,14 @@ module RD
     end
 
     def apply_to_DescListItem(element, term, contents)
+      term_attributes = {}
+      id = term_index_id(term)
+      unless @term_indexes[id]
+        @term_indexes[id] = term
+        term_attributes["id"] = "#{ref_entry_id}.#{id}"
+      end
       tag("varlistentry", {},
-          tag("term", {}, term),
+          tag("term", term_attributes, term),
           tag("listitem", {}, *contents))
     end
 
@@ -176,8 +244,12 @@ module RD
       tag("code", {}, *contents)
     end
 
+    def apply_to_Var(element, contents)
+      tag("varname", {}, *contents)
+    end
+
     def apply_to_StringElement(element)
-      apply_to_String(element.content.chomp)
+      apply_to_String(remove_newline(element.content))
     end
 
     def apply_to_String(element)
@@ -217,6 +289,11 @@ module RD
       remove_suffix(File.basename(filename)).downcase
     end
 
+    def term_index_id(term)
+      first_sentence = term.to_s.split(/\s/, 2)[0]
+      first_sentence.gsub(/[._]/, "-")
+    end
+
     def tag(name, attributes={}, *contents)
       encoded_attributes = attributes.collect do |key, value|
         "#{key}=\"#{h(value)}\""
@@ -264,10 +341,10 @@ module RD
     end
 
     def pre_collect_section_contents(contents)
-      section_contents = [[0, []]]
+      section_contents = [[0, [], nil]]
       contents.each do |content|
         if content.is_a?(Array) and content[0] == :headline
-          _, level, title = content
+          _, level, title, id = content
           title_tag = tag("title", {}, *title)
           sub_section_contents = []
           while section_contents.last[0] > level
@@ -276,7 +353,7 @@ module RD
                                              *sub_contents.flatten) + "\n")
           end
           section_contents.last[1].concat(sub_section_contents)
-          section_contents << [level, [title_tag]]
+          section_contents << [level, [title_tag], id]
         else
           section_contents.last[1] << content
         end
@@ -289,15 +366,19 @@ module RD
       section_contents.last[0].downto(0) do |level|
         sub_section_contents = []
         while section_contents.last[0] > level
-          sub_level, sub_contents = section_contents.pop
-          sub_section_contents.unshift(tag("refsect#{sub_level}", {},
+          sub_level, sub_contents, id = section_contents.pop
+          attributes = {}
+          attributes["id"] = id if id
+          sub_section_contents.unshift(tag("refsect#{sub_level}", attributes,
                                            *sub_contents))
         end
         unless sub_section_contents.empty?
           raise "!?" unless section_contents.last[0] == level
-          _, contents = section_contents.pop
+          _, contents, id = section_contents.pop
           if level > 0
-            contents = tag("refsect#{level}", {},
+            attributes = {}
+            attributes["id"] = id if id
+            contents = tag("refsect#{level}", attributes,
                            *(contents + sub_section_contents))
           else
             contents = sub_section_contents.join("\n\n")
@@ -312,12 +393,16 @@ module RD
       listitem.children.size == 1 and listitem.children[0].is_a?(TextBlock)
     end
 
+    def remove_hash_suffix(string)
+      string.sub(/\#.*\z/, "")
+    end
+
     def remove_lang_suffix(string)
       string.sub(/\.[a-z]{2}\z/, "")
     end
 
     def remove_suffix(string)
-      string.sub(/(?:\.[^.]+)+\z/, "")
+      string.sub(/(?:\.[^.]{2,4})+\z/, "")
     end
   end
 end
