@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007-2009  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2007-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -37,6 +37,9 @@
 #include <cutter/cut-test-context.h>
 #include <cutter/cut-backtrace-entry.h>
 #include <cutter/cut-verbose-level.h>
+#include <cutter/cut-colorize-differ.h>
+#include <cutter/cut-console-diff-writer.h>
+#include <cutter/cut-console.h>
 #include <cutter/cut-enum-types.h>
 
 #define CUT_TYPE_CONSOLE_UI            cut_type_console_ui
@@ -46,19 +49,9 @@
 #define CUT_IS_CONSOLE_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CUT_TYPE_CONSOLE_UI))
 #define CUT_CONSOLE_UI_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), CUT_TYPE_CONSOLE_UI, CutConsoleUIClass))
 
-#define RED_COLOR "\033[01;31m"
-#define RED_BACK_COLOR "\033[41m"
-#define GREEN_COLOR "\033[01;32m"
-#define GREEN_BACK_COLOR "\033[01;42m"
-#define YELLOW_COLOR "\033[01;33m"
-#define BLUE_COLOR "\033[01;34m"
-#define BLUE_BACK_COLOR "\033[01;44m"
-#define MAGENTA_COLOR "\033[01;35m"
-#define CYAN_COLOR "\033[01;36m"
-#define WHITE_COLOR "\033[01;37m"
-#define NORMAL_COLOR "\033[00m"
-
-#define CRASH_COLOR RED_BACK_COLOR WHITE_COLOR
+#define CRASH_COLOR                             \
+    CUT_CONSOLE_COLOR_RED_BACK                  \
+    CUT_CONSOLE_COLOR_WHITE
 
 typedef struct _CutConsoleUI CutConsoleUI;
 typedef struct _CutConsoleUIClass CutConsoleUIClass;
@@ -344,22 +337,22 @@ status_to_color(CutTestResultStatus status)
 
     switch (status) {
     case CUT_TEST_RESULT_SUCCESS:
-        color = GREEN_COLOR;
+        color = CUT_CONSOLE_COLOR_GREEN;
         break;
     case CUT_TEST_RESULT_NOTIFICATION:
-        color = CYAN_COLOR;
+        color = CUT_CONSOLE_COLOR_CYAN;
         break;
     case CUT_TEST_RESULT_OMISSION:
-        color = BLUE_COLOR;
+        color = CUT_CONSOLE_COLOR_BLUE;
         break;
     case CUT_TEST_RESULT_PENDING:
-        color = MAGENTA_COLOR;
+        color = CUT_CONSOLE_COLOR_MAGENTA;
         break;
     case CUT_TEST_RESULT_FAILURE:
-        color = RED_COLOR;
+        color = CUT_CONSOLE_COLOR_RED;
         break;
     case CUT_TEST_RESULT_ERROR:
-        color = YELLOW_COLOR;
+        color = CUT_CONSOLE_COLOR_YELLOW;
         break;
     case CUT_TEST_RESULT_CRASH:
         color = CRASH_COLOR;
@@ -379,7 +372,7 @@ print_with_colorv (CutConsoleUI *console, const gchar *color,
    if (console->use_color) {
         gchar *message;
         message = g_strdup_vprintf(format, args);
-        g_print("%s%s%s", color, message, NORMAL_COLOR);
+        g_print("%s%s%s", color, message, CUT_CONSOLE_COLOR_NORMAL);
         g_free(message);
     } else {
         g_vprintf(format, args);
@@ -436,9 +429,9 @@ cb_start_test_case (CutRunContext *run_context, CutTestCase *test_case,
     if (console->verbose_level < CUT_VERBOSE_LEVEL_VERBOSE)
         return;
 
-    print_with_color(console, GREEN_BACK_COLOR,
-                     "%s:", cut_test_get_name(CUT_TEST(test_case)));
-    g_print("\n");
+    print_with_color(console, CUT_CONSOLE_COLOR_GREEN_BACK,
+                     "%s", cut_test_get_name(CUT_TEST(test_case)));
+    g_print(":\n");
 }
 
 static void
@@ -450,9 +443,9 @@ cb_start_test_iterator (CutRunContext *run_context,
         return;
 
     g_print("  ");
-    print_with_color(console, BLUE_BACK_COLOR,
-                     "%s:", cut_test_get_name(CUT_TEST(test_iterator)));
-    g_print("\n");
+    print_with_color(console, CUT_CONSOLE_COLOR_BLUE_BACK,
+                     "%s", cut_test_get_name(CUT_TEST(test_iterator)));
+    g_print(":\n");
 }
 
 static void
@@ -684,7 +677,7 @@ print_each_attribute (gpointer key, gpointer value, gpointer data)
 
 static void
 print_test_attributes (CutConsoleUI *console, CutTestResultStatus status,
-                     CutTest *test)
+                       CutTest *test)
 {
     ConsoleAndStatus info;
     info.console = console;
@@ -693,6 +686,96 @@ print_test_attributes (CutConsoleUI *console, CutTestResultStatus status,
                          (GHFunc)print_each_attribute, &info);
 }
 
+static CutDiffWriter *
+console_diff_writer_new (gboolean use_color)
+{
+    CutDiffWriter *writer;
+    gchar *deleted_mark_color, *inserted_mark_color;
+    gchar *deleted_line_color, *inserted_line_color;
+    gchar *deleted_segment_color, *inserted_segment_color;
+
+    writer = cut_console_diff_writer_new(use_color);
+
+    deleted_mark_color =
+        g_strdup(cut_console_diff_writer_get_deleted_mark_color(writer));
+    inserted_mark_color =
+        g_strdup(cut_console_diff_writer_get_inserted_mark_color(writer));
+    deleted_line_color =
+        g_strdup(cut_console_diff_writer_get_deleted_line_color(writer));
+    inserted_line_color =
+        g_strdup(cut_console_diff_writer_get_inserted_line_color(writer));
+    deleted_segment_color =
+        g_strdup(cut_console_diff_writer_get_deleted_segment_color(writer));
+    inserted_segment_color =
+        g_strdup(cut_console_diff_writer_get_inserted_segment_color(writer));
+
+    cut_console_diff_writer_set_inserted_mark_color(writer, deleted_mark_color);
+    cut_console_diff_writer_set_deleted_mark_color(writer, inserted_mark_color);
+    cut_console_diff_writer_set_inserted_line_color(writer, deleted_line_color);
+    cut_console_diff_writer_set_deleted_line_color(writer, inserted_line_color);
+    cut_console_diff_writer_set_inserted_segment_color(writer,
+                                                       deleted_segment_color);
+    cut_console_diff_writer_set_deleted_segment_color(writer,
+                                                      inserted_segment_color);
+
+    g_free(deleted_mark_color);
+    g_free(inserted_mark_color);
+    g_free(deleted_line_color);
+    g_free(inserted_line_color);
+    g_free(deleted_segment_color);
+    g_free(inserted_segment_color);
+
+    return writer;
+}
+
+static void
+print_result_detail (CutConsoleUI *console, CutTestResultStatus status,
+                     CutTestResult *result)
+{
+    const gchar *expected, *actual;
+
+    expected = cut_test_result_get_expected(result);
+    actual = cut_test_result_get_actual(result);
+    if (console->use_color && expected && actual) {
+        const gchar *user_message, *system_message;
+        CutDiffer *differ;
+
+        user_message = cut_test_result_get_user_message(result);
+        system_message = cut_test_result_get_system_message(result);
+        if (user_message)
+            g_print("\n%s", user_message);
+        if (system_message)
+            g_print("\n%s", system_message);
+
+        g_print("\n");
+        g_print("expected: <");
+        print_for_status(console, CUT_TEST_RESULT_SUCCESS, "%s", expected);
+        g_print(">\n");
+        g_print("  actual: <");
+        print_for_status(console, CUT_TEST_RESULT_FAILURE, "%s", actual);
+        g_print(">");
+
+        differ = cut_colorize_differ_new(expected, actual);
+        if (cut_differ_need_diff(differ)) {
+            CutDiffWriter *writer;
+
+            g_print("\n\n");
+            g_print("diff:\n");
+            writer = console_diff_writer_new(console->use_color);
+            cut_differ_diff(differ, writer);
+            g_object_unref(writer);
+        }
+        g_object_unref(differ);
+    } else {
+        const gchar *message;
+
+        message = cut_test_result_get_message(result);
+        if (message) {
+            g_print("\n%s", message);
+        }
+    }
+}
+
 static void
 print_results (CutConsoleUI *console, CutRunContext *run_context)
 {
@@ -724,14 +807,12 @@ print_results (CutConsoleUI *console, CutRunContext *run_context)
         CutTestResultStatus status;
         CutTest *test;
         const GList *node;
-        const gchar *message;
         const gchar *name;
 
         status = cut_test_result_get_status(result);
         if (status == CUT_TEST_RESULT_SUCCESS)
             continue;
 
-        message = cut_test_result_get_message(result);
         name = cut_test_result_get_test_name(result);
         if (!name)
             name = cut_test_result_get_test_case_name(result);
@@ -739,17 +820,15 @@ print_results (CutConsoleUI *console, CutRunContext *run_context)
             name = cut_test_result_get_test_suite_name(result);
 
         g_print("\n%d) ", i);
-        print_for_status(console, status, "%s: %s",
-                         status_to_name(status), name);
+        print_for_status(console, status, "%s", status_to_name(status));
+        g_print(": %s", name);
 
         test = cut_test_result_get_test(result);
         if (test)
             print_test_attributes(console, status, test);
 
-        if (message) {
-            g_print("\n");
-            print_for_status(console, status, "%s", message);
-        }
+        print_result_detail(console, status, result);
+
         g_print("\n");
         for (node = cut_test_result_get_backtrace(result);
              node;
