@@ -15,6 +15,9 @@ void data_fixture_function (void);
 void test_fixture_function (gconstpointer data);
 void test_fail_to_load (void);
 void test_load_test_iterator (void);
+void test_load_cpp_namespace (void);
+void data_cpp_fixture_function (void);
+void test_cpp_fixture_function (gconstpointer data);
 
 static CutLoader *loader;
 static CutTestCase *test_case;
@@ -76,14 +79,6 @@ run (void)
                                          test_case);
 }
 
-static gchar *expected_functions[] = {
-    "test_abcdefghijklmnopqratuvwzyz_ABCDEFGHIJKLMNOPQRSTUVWXYZ_0123456789",
-    "test_stub_function1",
-    "test_stub_function2",
-    "test_stub_function3",
-    NULL
-};
-
 static gint
 compare_function_name (gconstpointer a, gconstpointer b)
 {
@@ -99,10 +94,17 @@ test_load_function (void)
 {
     CutTestContainer *container;
     GList *tests, *list;
-    const gchar *target_test_names[] = {"/.*/", NULL};
     gint i;
+    const gchar *target_test_names[] = {"/.*/", NULL};
+    gchar *expected_functions[] = {
+        "test_abcdefghijklmnopqratuvwzyz_ABCDEFGHIJKLMNOPQRSTUVWXYZ_0123456789",
+        "test_stub_function1",
+        "test_stub_function2",
+        "test_stub_function3",
+        NULL
+    };
 
-    loader = loader_new("test", "stub-loader-test." G_MODULE_SUFFIX);
+    loader = loader_new("test", "stub-test-functions." G_MODULE_SUFFIX);
     test_case = cut_loader_load_test_case(loader);
     cut_assert(test_case);
 
@@ -300,6 +302,161 @@ test_load_test_iterator (void)
     cut_assert_test_result_summary(run_context, 2, 2, 2, 0, 0, 0, 0, 0);
 }
 
+void
+test_load_cpp_namespace (void)
+{
+    CutTestContainer *container;
+    GList *tests, *list;
+    gint i;
+    const gchar *target_test_names[] = {"/.*/", NULL};
+    gchar *expected_functions[] = {
+        "calc::test_add",
+        "calc::test_sub",
+        NULL
+    };
+
+    loader = loader_new("cpp", "stub-namespace." G_MODULE_SUFFIX);
+    test_case = cut_loader_load_test_case(loader);
+    cut_assert(test_case);
+
+    container = CUT_TEST_CONTAINER(test_case);
+    cut_assert_equal_int(2,
+                         cut_test_container_get_n_tests(container, NULL));
+
+    run_context = cut_test_runner_new();
+    cut_run_context_set_target_test_names(run_context, target_test_names);
+    cut_assert_equal_int(2,
+                         cut_test_container_get_n_tests(container, run_context));
+
+    tests = (GList *)cut_test_container_get_children(container);
+    cut_assert(tests);
+
+    tests = g_list_sort(tests, compare_function_name);
+    test_names = g_new0(gchar *, g_list_length(tests) + 1);
+    for (list = tests, i = 0; list; list = g_list_next(list), i++) {
+        CutTest *test;
+
+        cut_assert(list->data);
+        cut_assert(CUT_IS_TEST(list->data));
+
+        test = CUT_TEST(list->data);
+        test_names[i] = g_strdup(cut_test_get_name(test));
+    }
+    cut_assert_equal_string_array(expected_functions, test_names);
+}
+
+static const gchar *
+mangle (const gchar *component, ...)
+{
+    GString *mangled;
+    va_list args;
+
+    mangled = g_string_new("_ZN");
+    va_start(args, component);
+    while (component) {
+        g_string_append_printf(mangled,
+                               "%" G_GSIZE_FORMAT "%s",
+                               strlen(component), component);
+        component = va_arg(args, gchar *);
+    }
+    va_end(args);
+    g_string_append(mangled, "Ev");
+
+    return cut_take_string(g_string_free(mangled, FALSE));
+}
+
+void
+data_cpp_fixture_function (void)
+{
+    cut_add_data("without prefix",
+                 fixture_test_data_new("without-prefix." G_MODULE_SUFFIX,
+                                       "startup",
+                                       "setup",
+                                       "teardown",
+                                       "shutdown"),
+                 fixture_test_data_free,
+                 "with prefix",
+                 fixture_test_data_new("with-prefix." G_MODULE_SUFFIX,
+                                       "cut_startup",
+                                       "cut_setup",
+                                       "cut_teardown",
+                                       "cut_shutdown"),
+                 fixture_test_data_free,
+                 "all",
+                 fixture_test_data_new("all." G_MODULE_SUFFIX,
+                                       "cut_startup",
+                                       "cut_setup",
+                                       "cut_teardown",
+                                       "cut_shutdown"),
+                 fixture_test_data_free);
+}
+
+void
+test_cpp_fixture_function (gconstpointer data)
+{
+    const FixtureTestData *test_data = data;
+    CutStartupFunction expected_startup_function = NULL;
+    CutStartupFunction actual_startup_function = NULL;
+    CutShutdownFunction expected_shutdown_function = NULL;
+    CutShutdownFunction actual_shutdown_function = NULL;
+    CutSetupFunction expected_setup_function = NULL;
+    CutSetupFunction actual_setup_function = NULL;
+    CutTeardownFunction expected_teardown_function = NULL;
+    CutTeardownFunction actual_teardown_function = NULL;
+    gchar *so_filename;
+
+    loader = loader_new("cpp-fixture", test_data->file_name);
+    test_case = cut_loader_load_test_case(loader);
+    cut_assert(test_case);
+
+    g_object_get(G_OBJECT(loader),
+                 "so-filename", &so_filename,
+                 NULL);
+    module = g_module_open(so_filename,
+                           G_MODULE_BIND_LAZY | G_MODULE_BIND_LOCAL);
+    g_free(so_filename);
+    cut_assert_not_null(module);
+    if (test_data->startup_function_name)
+        cut_assert_true(g_module_symbol(module,
+                                        mangle("fixture",
+                                               test_data->startup_function_name,
+                                               NULL),
+                                        (gpointer)&expected_startup_function));
+    if (test_data->setup_function_name)
+        cut_assert_true(g_module_symbol(module,
+                                        mangle("fixture",
+                                               test_data->setup_function_name,
+                                               NULL),
+                                        (gpointer)&expected_setup_function));
+    if (test_data->teardown_function_name)
+        cut_assert_true(g_module_symbol(module,
+                                        mangle("fixture",
+                                               test_data->teardown_function_name,
+                                               NULL),
+                                        (gpointer)&expected_teardown_function));
+    if (test_data->shutdown_function_name)
+        cut_assert_true(g_module_symbol(module,
+                                        mangle("fixture",
+                                               test_data->shutdown_function_name,
+                                               NULL),
+                                        (gpointer)&expected_shutdown_function));
+
+    g_object_get(G_OBJECT(test_case),
+                 "startup-function", &actual_startup_function,
+                 "setup-function", &actual_setup_function,
+                 "teardown-function", &actual_teardown_function,
+                 "shutdown-function", &actual_shutdown_function,
+                 NULL);
+    cut_assert_equal_pointer(expected_startup_function,
+                             actual_startup_function);
+    cut_assert_equal_pointer(expected_setup_function,
+                             actual_setup_function);
+    cut_assert_equal_pointer(expected_teardown_function,
+                             actual_teardown_function);
+    cut_assert_equal_pointer(expected_shutdown_function,
+                             actual_shutdown_function);
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
