@@ -93,6 +93,7 @@ cut_setup (void)
         "  --fatal-failures                                Treat failures as fatal problem" LINE_FEED_CODE
         "  --keep-opening-modules                          Keep opening loaded modules to resolve symbols for debugging" LINE_FEED_CODE
         "  --enable-convenience-attribute-definition       Enable convenience but danger '#{ATTRIBUTE_NAME}_#{TEST_NAME - 'test_' PREFIX}' attribute set function" LINE_FEED_CODE
+        "  --stop-before-test                              Set breakpoints at each line which invokes test. You can step into a test function with your debugger easily." LINE_FEED_CODE
         "" LINE_FEED_CODE,
         g_get_prgname(),
         g_get_prgname());
@@ -207,7 +208,9 @@ test_help_all (void)
         "  --stream-log-directory=DIRECTORY                Obsoleted. Use --stream-directory instead" LINE_FEED_CODE
         "" LINE_FEED_CODE
         "Report Options" LINE_FEED_CODE
+#if defined(HAVE_PANGO_CAIRO) && defined(HAVE_CAIRO_PDF)
         "  --pdf-report=FILE                               Set filename of pdf report" LINE_FEED_CODE
+#endif
         "  --xml-report=FILE                               Set filename of xml report" LINE_FEED_CODE
         "" LINE_FEED_CODE
         "UI Options" LINE_FEED_CODE
@@ -262,6 +265,7 @@ test_help_all (void)
         "  --fatal-failures                                Treat failures as fatal problem" LINE_FEED_CODE
         "  --keep-opening-modules                          Keep opening loaded modules to resolve symbols for debugging" LINE_FEED_CODE
         "  --enable-convenience-attribute-definition       Enable convenience but danger '#{ATTRIBUTE_NAME}_#{TEST_NAME - 'test_' PREFIX}' attribute set function" LINE_FEED_CODE
+        "  --stop-before-test                              Set breakpoints at each line which invokes test. You can step into a test function with your debugger easily." LINE_FEED_CODE
 #ifdef HAVE_GTK
         "  --display=DISPLAY                               X display to use" LINE_FEED_CODE
 #endif
